@@ -39,7 +39,8 @@ struct _PangoFontDescription
 
   guint16 mask;
   guint static_family : 1;
-
+  guint size_is_absolute : 1;
+	
   int size;
 };
 
@@ -76,6 +77,7 @@ pango_font_description_new (void)
   desc->weight = PANGO_WEIGHT_NORMAL;
   desc->stretch = PANGO_STRETCH_NORMAL;
   desc->size = 0;
+  desc->size_is_absolute = FALSE;
 
   return desc;
 }
@@ -331,10 +333,16 @@ pango_font_description_get_stretch (const PangoFontDescription *desc)
 /**
  * pango_font_description_set_size:
  * @desc: a #PangoFontDescription
- * @size: the size for the font description in pango units.
- *        There are #PANGO_SCALE Pango units in one device unit (device unit is a point, for font sizes).
+ * @size: the size of the font in points, scaled by PANGO_SCALE. (That is,
+ *        a @size value of 10 * PANGO_SCALE is a 10 point font. The conversion
+ *        factor between points and device units depends on system configuration
+ *        and the output device. For screen display, a logical DPI of 96 is
+ *        common, in which case a 10 point font corresponds to a 10 * (96 / 72) = 13.3
+ *        pixel font. Use pango_font_description_set_absolute_size() if you need
+ *        a particular size in device units.
  * 
- * Sets the size field of a font description.
+ * Sets the size field of a font description in fractional points. This is mutually
+ * exclusive with pango_font_description_set_absolute_size(). 
  **/
 void
 pango_font_description_set_size (PangoFontDescription *desc,
@@ -343,6 +351,7 @@ pango_font_description_set_size (PangoFontDescription *desc,
   g_return_if_fail (desc != NULL);
   
   desc->size = size;
+  desc->size_is_absolute = FALSE;
   desc->mask |= PANGO_FONT_MASK_SIZE;
 }
 
@@ -353,9 +362,9 @@ pango_font_description_set_size (PangoFontDescription *desc,
  * Gets the size field of a font description.
  * See pango_font_description_get_size().
  * 
- * Return value: the size field for the font description in pango
- *   units.  (There are #PANGO_SCALE pango units in one device unit &mdash;
- *   for fonts, font points are the device unit.) Returns 0 if the stretch field has not
+ * Return value: the size field for the font description in points or device units.
+ *   You must call pango_font_description_get_size_is_absolute()
+ *   to find out which is the case. Returns 0 if the size field has not
  *   previously been set.  pango_font_description_get_set_fields() to
  *   find out if the field was explicitely set or not.
  **/
@@ -367,6 +376,50 @@ pango_font_description_get_size (const PangoFontDescription *desc)
   return desc->size;
 }
 
+/**
+ * pango_font_description_set_absolute_size:
+ * @desc: a #PangoFontDescription
+ * @size: the new size, in pango units. There are #PANGO_SCALE Pango units in one
+ *   device unit. For an output backend where a device unit is a pixel, a @size
+ *   value of 10 * PANGO_SCALE gives a 10 pixel font.
+ * 
+ * Sets the size field of a font description, in device units. This is mutually
+ * exclusive with pango_font_description_set_size().
+ *
+ * Since: 1.8
+ **/
+void
+pango_font_description_set_absolute_size (PangoFontDescription *desc,
+					  double                size)
+{
+  g_return_if_fail (desc != NULL);
+  
+  desc->size = size;
+  desc->size_is_absolute = TRUE;
+  desc->mask |= PANGO_FONT_MASK_SIZE;
+}
+
+/**
+ * pango_font_description_get_size_is_absolute:
+ * @desc: a #PangoFontDescription
+ * 
+ * Determines whether the size of the font is in points or device units.
+ * See pango_font_description_set_size() and pango_font_description_set_absolute_size().
+ * 
+ * Return value: whether the the size for the font description is in
+ *   points or device units.  Use pango_font_description_get_set_fields() to
+ *   find out if the size field of the font description was explicitely set or not.
+ *
+ * Since: 1.8
+ **/
+gboolean
+pango_font_description_get_size_is_absolute (const PangoFontDescription *desc)
+{
+  g_return_val_if_fail (desc != NULL, FALSE);
+
+  return desc->size_is_absolute;
+}
+
 /**
  * pango_font_description_get_set_fields:
  * @desc: a #PangoFontDescription
@@ -443,7 +496,10 @@ pango_font_description_merge (PangoFontDescription       *desc,
   if (new_mask & PANGO_FONT_MASK_STRETCH)
     desc->stretch = desc_to_merge->stretch;
   if (new_mask & PANGO_FONT_MASK_SIZE)
-    desc->size = desc_to_merge->size;
+    {
+      desc->size = desc_to_merge->size;
+      desc->size_is_absolute = desc_to_merge->size_is_absolute;
+    }
 
   desc->mask |= new_mask;
 }
@@ -486,7 +542,10 @@ pango_font_description_merge_static (PangoFontDescription       *desc,
   if (new_mask & PANGO_FONT_MASK_STRETCH)
     desc->stretch = desc_to_merge->stretch;
   if (new_mask & PANGO_FONT_MASK_SIZE)
-    desc->size = desc_to_merge->size;
+    {
+      desc->size = desc_to_merge->size;
+      desc->size_is_absolute = desc_to_merge->size_is_absolute;
+    }
 
   desc->mask |= new_mask;
 }
@@ -614,6 +673,7 @@ pango_font_description_equal (const PangoFontDescription  *desc1,
 	  desc1->weight == desc2->weight &&
 	  desc1->stretch == desc2->stretch &&
 	  desc1->size == desc2->size &&
+	  desc1->size_is_absolute == desc2->size_is_absolute &&
 	  (desc1->family_name == desc2->family_name ||
 	   (desc1->family_name && desc2->family_name && g_ascii_strcasecmp (desc1->family_name, desc2->family_name) == 0)));
 }
@@ -655,7 +715,10 @@ pango_font_description_hash (const PangoFontDescription *desc)
   if (desc->mask & PANGO_FONT_MASK_FAMILY)
     hash = case_insensitive_hash (desc->family_name);
   if (desc->mask & PANGO_FONT_MASK_SIZE)
-    hash ^= desc->size;
+    {
+      hash ^= desc->size;
+      hash ^= desc->size_is_absolute ? 0xc33ca55a : 0;
+    }
   if (desc->mask & PANGO_FONT_MASK_STYLE)
     hash ^= desc->style << 16;
   if (desc->mask & PANGO_FONT_MASK_VARIANT)
@@ -713,29 +776,29 @@ typedef struct
   const char *str;
 } FieldMap;
 
-static FieldMap style_map[] = {
+static const FieldMap style_map[] = {
   { PANGO_STYLE_NORMAL, NULL },
   { PANGO_STYLE_OBLIQUE, "Oblique" },
   { PANGO_STYLE_ITALIC, "Italic" }
 };
 
-static FieldMap variant_map[] = {
+static const FieldMap variant_map[] = {
   { PANGO_VARIANT_NORMAL, NULL },
   { PANGO_VARIANT_SMALL_CAPS, "Small-Caps" }
 };
 
-static FieldMap weight_map[] = {
+static const FieldMap weight_map[] = {
   { PANGO_WEIGHT_ULTRALIGHT, "Ultra-Light" },
   { PANGO_WEIGHT_LIGHT, "Light" },
   { PANGO_WEIGHT_NORMAL, NULL },
   { 500, "Medium" },
-  { 600, "Semi-Bold" },
+  { PANGO_WEIGHT_SEMIBOLD, "Semi-Bold" },
   { PANGO_WEIGHT_BOLD, "Bold" },
   { PANGO_WEIGHT_ULTRABOLD, "Ultra-Bold" },
   { PANGO_WEIGHT_HEAVY, "Heavy" }
 };
 
-static FieldMap stretch_map[] = {
+static const FieldMap stretch_map[] = {
   { PANGO_STRETCH_ULTRA_CONDENSED, "Ultra-Condensed" },
   { PANGO_STRETCH_EXTRA_CONDENSED, "Extra-Condensed" },
   { PANGO_STRETCH_CONDENSED,       "Condensed" },
@@ -748,7 +811,7 @@ static FieldMap stretch_map[] = {
 };
 
 static gboolean
-find_field (FieldMap *map, int n_elements, const char *str, int len, int *val)
+find_field (const FieldMap *map, int n_elements, const char *str, int len, int *val)
 {
   int i;
   
@@ -796,6 +859,25 @@ getword (const char *str, const char *last, size_t *wordlen)
   return result;
 }
 
+static gboolean
+parse_size (const char *word,
+	    size_t      wordlen,
+	    int        *pango_size)
+{
+  char *end;
+  double size = g_ascii_strtod (word, &end);
+
+  if (end - word == wordlen) /* word is a valid float */
+    {
+      if (pango_size)
+	*pango_size = (int)(size * PANGO_SCALE + 0.5);
+
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
 /**
  * pango_font_description_from_string:
  * @str: string representation of a font description.
@@ -839,6 +921,7 @@ pango_font_description_from_string (const char *str)
   desc->stretch = PANGO_STRETCH_NORMAL;
   
   desc->size = 0;
+  desc->size_is_absolute = FALSE;
 
   len = strlen (str);
   last = str + len;
@@ -848,13 +931,10 @@ pango_font_description_from_string (const char *str)
    */
   if (wordlen != 0)
     {
-      char *end;
-      double size = g_ascii_strtod (p, &end);
-      if (end - p == wordlen) /* word is a valid float */
+      if (parse_size (p, wordlen, &desc->size))
 	{
-	  desc->size = (int)(size * PANGO_SCALE + 0.5);
 	  desc->mask |= PANGO_FONT_MASK_SIZE;
-	  last = p;	  
+	  last = p;
 	}
     }
 
@@ -897,7 +977,7 @@ pango_font_description_from_string (const char *str)
 }
 
 static void
-append_field (GString *str, FieldMap *map, int n_elements, int val)
+append_field (GString *str, const FieldMap *map, int n_elements, int val)
 {
   int i;
   for (i=0; i<n_elements; i++)
@@ -942,9 +1022,20 @@ pango_font_description_to_string (const PangoFontDescription  *desc)
       size_t wordlen;
 
       g_string_append (result, desc->family_name);
-      
+
+      /* We need to add a trailing comma if the family name ends
+       * in a keyword like "Bold", or if the family name ends in
+       * a number and no keywords will be added.
+       */
       p = getword (desc->family_name, desc->family_name + strlen(desc->family_name), &wordlen);
-      if (wordlen != 0 && find_field_any (p, wordlen, NULL))
+      if (wordlen != 0 &&
+	  (find_field_any (p, wordlen, NULL) ||
+	   (parse_size (p, wordlen, NULL) &&
+	    desc->weight == PANGO_WEIGHT_NORMAL &&
+	    desc->style == PANGO_STYLE_NORMAL &&
+	    desc->stretch == PANGO_STRETCH_NORMAL &&
+	    desc->variant == PANGO_VARIANT_NORMAL &&
+	    (desc->mask & PANGO_FONT_MASK_SIZE) == 0)))
 	g_string_append_c (result, ',');
     }
 
@@ -999,33 +1090,17 @@ pango_font_description_to_filename (const PangoFontDescription  *desc)
 
   return result;
 }  
-  
-GType
-pango_font_get_type (void)
-{
-  static GType object_type = 0;
 
-  if (!object_type)
-    {
-      static const GTypeInfo object_info =
-      {
-        sizeof (PangoFontClass),
-        (GBaseInitFunc) NULL,
-        (GBaseFinalizeFunc) NULL,
-        NULL,           /* class_init */
-        NULL,           /* class_finalize */
-        NULL,           /* class_data */
-        sizeof (PangoFont),
-        0,              /* n_preallocs */
-	NULL            /* init */
-      };
-      
-      object_type = g_type_register_static (G_TYPE_OBJECT,
-                                            "PangoFont",
-                                            &object_info, 0);
-    }
-  
-  return object_type;
+G_DEFINE_TYPE (PangoFont, pango_font, G_TYPE_OBJECT)
+
+static void
+pango_font_class_init (PangoFontClass *class)
+{
+}
+
+static void
+pango_font_init (PangoFont *font)
+{
 }
 
 /**
@@ -1100,7 +1175,7 @@ pango_font_find_shaper (PangoFont     *font,
  * coordinate system for each rectangle has its origin at the
  * base line and horizontal origin of the character with increasing
  * coordinates extending to the right and down. The macros PANGO_ASCENT(),
- * PANGO_DESCENT(), PANGO_LBEARING(), and PANGO_RBEARING can be used to convert
+ * PANGO_DESCENT(), PANGO_LBEARING(), and PANGO_RBEARING() can be used to convert
  * from the extents rectangle to more traditional font metrics. The units
  * of the rectangles are in 1/PANGO_SCALE of a device unit.
  **/
@@ -1127,7 +1202,7 @@ pango_font_get_glyph_extents  (PangoFont      *font,
  * be provided to indicate that the metrics should be retrieved that
  * correspond to the script(s) used by that language.
  *
- * Returns: a #PangoMetrics object. The caller must call pango_font_metrics_unref()
+ * Returns: a #PangoFontMetrics object. The caller must call pango_font_metrics_unref()
  *   when finished using the object.
  **/
 PangoFontMetrics *
@@ -1288,36 +1363,98 @@ pango_font_metrics_get_approximate_digit_width (PangoFontMetrics *metrics)
   return metrics->approximate_digit_width;
 }
 
+/**
+ * pango_font_metrics_get_underline_position:
+ * @metrics: a #PangoFontMetrics structure
+ * 
+ * Gets the suggested position to draw the underline.
+ * The value returned is the distance <emphasis>above</emphasis> the
+ * baseline of the top of the underline. Since most fonts have
+ * underline positions beneath the baseline, this value is typically
+ * negative.
+ * 
+ * Return value: the suggested underline position, in Pango units.
+ *
+ * Since: 1.6
+ **/
+int
+pango_font_metrics_get_underline_position (PangoFontMetrics *metrics)
+{
+  g_return_val_if_fail (metrics != NULL, 0);
+
+  return metrics->underline_position;
+}
+
+/**
+ * pango_font_metrics_get_underline_thickness:
+ * @metrics: a #PangoFontMetrics structure
+ * 
+ * Gets the suggested thickness to draw for the underline.
+ * 
+ * Return value: the suggested underline thickness, in Pango units.
+ *
+ * Since: 1.6
+ **/
+int
+pango_font_metrics_get_underline_thickness (PangoFontMetrics *metrics)
+{
+  g_return_val_if_fail (metrics != NULL, 0);
+
+  return metrics->underline_thickness;
+}
+
+/**
+ * pango_font_metrics_get_strikethrough_position:
+ * @metrics: a #PangoFontMetrics structure
+ * 
+ * Gets the suggested position to draw the strikethrough.
+ * The value returned is the distance <emphasis>above</emphasis> the
+ * baseline of the top of the strikethrough.
+ * 
+ * Return value: the suggested strikethrough position, in Pango units.
+ *
+ * Since: 1.6
+ **/
+int
+pango_font_metrics_get_strikethrough_position (PangoFontMetrics *metrics)
+{
+  g_return_val_if_fail (metrics != NULL, 0);
+
+  return metrics->strikethrough_position;
+}
+
+/**
+ * pango_font_metrics_get_strikethrough_thickness:
+ * @metrics: a #PangoFontMetrics structure
+ * 
+ * Gets the suggested thickness to draw for the strikethrough.
+ * 
+ * Return value: the suggested strikethrough thickness, in Pango units.
+ *
+ * Since: 1.6
+ **/
+int
+pango_font_metrics_get_strikethrough_thickness (PangoFontMetrics *metrics)
+{
+  g_return_val_if_fail (metrics != NULL, 0);
+
+  return metrics->strikethrough_thickness;
+}
+
 /*
  * PangoFontFamily
  */
 
-GType
-pango_font_family_get_type (void)
-{
-  static GType object_type = 0;
+G_DEFINE_TYPE (PangoFontFamily, pango_font_family, G_TYPE_OBJECT)
 
-  if (!object_type)
-    {
-      static const GTypeInfo object_info =
-      {
-        sizeof (PangoFontFamilyClass),
-        (GBaseInitFunc) NULL,
-        (GBaseFinalizeFunc) NULL,
-        NULL,           /* class_init */
-        NULL,           /* class_finalize */
-        NULL,           /* class_data */
-        sizeof (PangoFontFamily),
-        0,              /* n_preallocs */
-	NULL            /* init */
-      };
-      
-      object_type = g_type_register_static (G_TYPE_OBJECT,
-                                            "PangoFontFamily",
-                                            &object_info, 0);
-    }
-  
-  return object_type;
+static void
+pango_font_family_class_init (PangoFontFamilyClass *class)
+{
+}
+
+static void
+pango_font_family_init (PangoFontFamily *family)
+{
 }
 
 /**
@@ -1361,36 +1498,50 @@ pango_font_family_list_faces (PangoFontFamily  *family,
   PANGO_FONT_FAMILY_GET_CLASS (family)->list_faces (family, faces, n_faces);
 }
 
+/**
+ * pango_font_family_is_monospace:
+ * @family: a #PangoFontFamily
+ *
+ * A monospace font is a font designed for text display where the the
+ * characters form a regular grid. For Western languages this would
+ * mean that the advance width of all characters are the same, but
+ * this categorization also includes Asian fonts which include
+ * double-width characters: characters that occupy two grid cells.
+ * g_unichar_iswide() returns a result that indicates whether a
+ * character is typically double-width in a monospace font.
+ * 
+ * The best way to find out the grid-cell size is to call
+ * pango_font_metrics_get_approximate_digit_width(), since the results
+ * of pango_font_metrics_get_approximate_char_width() may be affected
+ * by double-width characters.  
+ * 
+ * Return value: TRUE if the family is monospace.
+ **/
+gboolean 
+pango_font_family_is_monospace (PangoFontFamily  *family)
+{
+  g_return_val_if_fail (PANGO_IS_FONT_FAMILY (family), FALSE);
+
+  if (PANGO_FONT_FAMILY_GET_CLASS (family)->is_monospace)
+    return PANGO_FONT_FAMILY_GET_CLASS (family)->is_monospace (family);
+  else
+    return FALSE;
+}
+
 /*
  * PangoFontFace
  */
 
-GType
-pango_font_face_get_type (void)
-{
-  static GType object_type = 0;
+G_DEFINE_TYPE (PangoFontFace, pango_font_face, G_TYPE_OBJECT)
 
-  if (!object_type)
-    {
-      static const GTypeInfo object_info =
-      {
-        sizeof (PangoFontFaceClass),
-        (GBaseInitFunc) NULL,
-        (GBaseFinalizeFunc) NULL,
-        NULL,           /* class_init */
-        NULL,           /* class_finalize */
-        NULL,           /* class_data */
-        sizeof (PangoFont),
-        0,              /* n_preallocs */
-	NULL            /* init */
-      };
-      
-      object_type = g_type_register_static (G_TYPE_OBJECT,
-                                            "PangoFontFace",
-                                            &object_info, 0);
-    }
-  
-  return object_type;
+static void
+pango_font_face_class_init (PangoFontFaceClass *class)
+{
+}
+
+static void
+pango_font_face_init (PangoFontFace *face)
+{
 }
 
 /**
@@ -1432,3 +1583,37 @@ pango_font_face_get_face_name (PangoFontFace *face)
 
   return PANGO_FONT_FACE_GET_CLASS (face)->get_face_name (face);
 }
+
+/**
+ * pango_font_face_list_sizes:
+ * @face: a #PangoFontFace.
+ * @sizes: location to store a pointer to an array of int. This array
+ *         should be freed with g_free().
+ * @n_sizes: location to store the number of elements in @sizes
+ * 
+ * List the available sizes for a font. This is only applicable to bitmap
+ * fonts. For scalable fonts, stores %NULL at the location pointed to by
+ * @sizes and 0 at the location pointed to by @n_sizes. The sizes returned
+ * are in pango units and are sorted in ascending order.
+ *
+ * Since: 1.4
+ **/
+void
+pango_font_face_list_sizes (PangoFontFace  *face, 
+                            int           **sizes, 
+                            int            *n_sizes)
+{
+  g_return_if_fail (PANGO_IS_FONT_FACE (face));
+  g_return_if_fail (sizes == NULL || n_sizes != NULL);
+
+  if (n_sizes == NULL)
+    return;
+
+  if (PANGO_FONT_FACE_GET_CLASS (face)->list_sizes != NULL)
+    PANGO_FONT_FACE_GET_CLASS (face)->list_sizes (face, sizes, n_sizes);
+  else
+    {
+      *sizes = NULL;
+      *n_sizes = 0;
+    }
+}
