@@ -96,7 +96,7 @@ struct _PangoXSizeInfo
   GSList *xlfds;
 };
 
-const struct {
+static const struct {
   const gchar *text;
   PangoWeight value;
 } weights_map[] = {
@@ -113,7 +113,7 @@ const struct {
   { "black",     900 }
 };
 
-const struct {
+static const struct {
   const gchar *text;
   PangoStyle value;
 } styles_map[] = {
@@ -122,7 +122,7 @@ const struct {
   { "o", PANGO_STYLE_OBLIQUE }
 };
 
-const struct {
+static const struct {
   const gchar *text;
   PangoStretch value;
 } stretches_map[] = {
@@ -1328,14 +1328,6 @@ pango_x_make_matching_xlfd (PangoFontMap *fontmap, char *xlfd, const char *chars
   return result;
 }
 
-static void
-free_coverages_foreach (gpointer key,
-			gpointer value,
-			gpointer data)
-{
-  pango_coverage_unref (value);
-}
-
 /**
  * pango_x_font_map_get_font_cache:
  * @font_map: a #PangoXFontMap.
